"use strict";
exports.__esModule = true;
var assert_1 = require("assert");
var readline_1 = require("readline");
var BRLInterface = /** @class */ (function () {
    function BRLInterface(stdout, stderr, stdin, completer) {
        if (stdout === void 0) { stdout = process.stdout; }
        if (stderr === void 0) { stderr = process.stderr; }
        if (stdin === void 0) { stdin = process.stdin; }
        if (completer === void 0) { completer = undefined; }
        this.stdout = stdout;
        this.stderr = stderr;
        this.state = InterfaceState.STOPPED;
        this.console = console;
        this.readline = readline_1.createInterface(stdin, stdout, completer);
        this.wrappedStdout = this.wrapWritable(stdout);
        this.wrappedStderr = this.wrapWritable(stderr);
        this.wrappedConsole = new console.Console(this.wrappedStdout, this.wrappedStderr);
    }
    BRLInterface.prototype.wrapWritable = function (writable) {
        var _this = this;
        var wrapped = Object.create(writable);
        wrapped.write = function (buffer, cb) {
            writable.write('\x1b[2K\r');
            var result = writable.write(buffer, cb);
            _this.readline._refreshLine();
            return result;
        };
        return wrapped;
    };
    BRLInterface.prototype.setPrompt = function (prompt) {
        this.readline.setPrompt(prompt);
    };
    BRLInterface.prototype.onLine = function (cb) {
        this.readline.on('line', cb);
    };
    BRLInterface.prototype.prompt = function () {
        this.readline.prompt();
    };
    BRLInterface.prototype.promptLoop = function (cb) {
        var _this = this;
        if (cb)
            this.onLine(cb);
        this.readline.on('line', function () { return _this.prompt(); });
        if (this.state !== InterfaceState.STARTED)
            this.start();
        this.prompt();
    };
    // The following methods allow the replacement of `process.stdout` and 
    // `process.stderr`. They will have an inconsistent behavior if used with 
    // other streams
    /**
     * Start replacing `process.stdout`, `process.stderr` and `console` with
     * their wrapped version
     */
    BRLInterface.prototype.start = function () {
        assert_1.ok(this.state === InterfaceState.STOPPED, 'interface already started');
        this.stdoutPD = Object.getOwnPropertyDescriptor(process, 'stdout');
        this.stderrPD = Object.getOwnPropertyDescriptor(process, 'stderr');
        this.consolePD = Object.getOwnPropertyDescriptor(global, 'console');
        var this$Interface = this;
        var cases = [
            [process, 'stdout', this.stdout, this.wrappedStdout],
            [process, 'stderr', this.stderr, this.wrappedStderr],
            [global, 'console', this.console, this.wrappedConsole],
        ];
        var _loop_1 = function (obj, name, base, wrapped) {
            Object.defineProperty(obj, name, {
                get: function () {
                    // If paused, return the base object
                    if (this$Interface.state === InterfaceState.PAUSED) {
                        return base;
                    }
                    // If started, return the wrapped object
                    else {
                        return wrapped;
                    }
                }
            });
        };
        for (var _i = 0, cases_1 = cases; _i < cases_1.length; _i++) {
            var _a = cases_1[_i], obj = _a[0], name = _a[1], base = _a[2], wrapped = _a[3];
            _loop_1(obj, name, base, wrapped);
        }
        this.state = InterfaceState.STARTED;
    };
    /**
     * Makes the replaced `process.stdout`, `process.stderr` and `console`
     * return their initial value until {@link resume} is called. This doesn't
     * reset the replaced properties, it just turns a flag on
     */
    BRLInterface.prototype.pause = function () {
        assert_1.ok(this.state === InterfaceState.STARTED, 'interface stopped or already paused');
        this.state = InterfaceState.PAUSED;
    };
    /**
     * @see {@link pause}
     */
    BRLInterface.prototype.resume = function () {
        assert_1.ok(this.state === InterfaceState.PAUSED, 'interface not paused');
        this.state = InterfaceState.STARTED;
    };
    /**
     * Completely reset `process.stdout`, `process.stderr` and `console` to
     * their original property descriptors
     */
    BRLInterface.prototype.stop = function () {
        assert_1.ok(this.state !== InterfaceState.STOPPED, 'interface already stopped');
        // Reset properties to their initial property descriptor
        Object.defineProperty(process, 'stdout', this.stdoutPD);
        Object.defineProperty(process, 'stderr', this.stderrPD);
        Object.defineProperty(global, 'console', this.consolePD);
        this.state = InterfaceState.STOPPED;
    };
    return BRLInterface;
}());
exports.BRLInterface = BRLInterface;
function createInterface(input, output, completer) {
    var iface = new BRLInterface(output, process.stderr, input, completer);
    iface.start();
    iface.readline.on('close', function () { return iface.stop(); });
    return iface.readline;
}
exports.createInterface = createInterface;
var InterfaceState;
(function (InterfaceState) {
    InterfaceState[InterfaceState["STARTED"] = 0] = "STARTED";
    InterfaceState[InterfaceState["PAUSED"] = 1] = "PAUSED";
    InterfaceState[InterfaceState["STOPPED"] = 2] = "STOPPED";
})(InterfaceState = exports.InterfaceState || (exports.InterfaceState = {}));
